﻿// -*- C++ -*-
/*!
 * @file  Motion.cpp
 * @brief Motion Comp
 * @date $Date$
 *
 * $Id$
 */

#include "Motion.h"

// Module specification
// <rtc-template block="module_spec">
static const char* motion_spec[] =
  {
    "implementation_id", "Motion",
    "type_name",         "Motion",
    "description",       "Motion Comp",
    "version",           "1.0.0",
    "vendor",            "group4",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
Motion::Motion(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Direction2In("Direction", m_Direction2),
    m_stickIn("Joystick", m_stick),
    m_LRF2In("LRF", m_LRF2),
    m_moveOut("Move", m_move)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
Motion::~Motion()
{
}



RTC::ReturnCode_t Motion::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("Direction", m_Direction2In);
  addInPort("Joystick", m_stickIn);
  addInPort("LRF", m_LRF2In);
  
  // Set OutPort buffer
  addOutPort("Move", m_moveOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // </rtc-template>
  


  m_move.data.va = 0.0;
  m_move.data.vx = 0.0;
  m_move.data.vy = 0.0;




  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Motion::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Motion::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Motion::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Motion::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t Motion::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t Motion::onExecute(RTC::UniqueId ec_id)
{
	if (m_stickIn.isNew()) {
		m_stickIn.read();
	}
	if (m_Direction2In.isNew()) {
		m_Direction2In.read();
	}
	if (m_LRF2In.isNew()) {
		m_LRF2In.read();
	}

	
	if (m_LRF2.data.va > 0.01 || m_LRF2.data.va < -0.01 || m_LRF2.data.vx >0.01 || m_LRF2.data.vx < -0.01 || m_LRF2.data.vy >0.01 || m_LRF2.data.vy < -0.01) {
		m_move.data.va = 0;
		m_move.data.vy = 0;
		m_move.data.vx = 0;
	}
	else {



		if (m_stick.data.va > 0.05 || m_stick.data.va < -0.05 || m_stick.data.vx >0.05 || m_stick.data.vx < -0.05 || m_stick.data.vy >0.05 || m_stick.data.vy < -0.05) {
			m_move.data.va = m_stick.data.va / 4;
			m_move.data.vy = m_stick.data.vy / 4;
			m_move.data.vx = m_stick.data.vx / 4;


		}
		else if (m_Direction2.data == -1.0 || m_Direction2.data == 1.0 || m_Direction2.data == 0.0)
		{
			m_move.data.vx = 0.08;
			m_move.data.va = 0.0;
			m_move.data.vy = 0.0;

		}
		else {
			m_move.data.vx = 0.0;
			m_move.data.va = 0.0;
			m_move.data.vy = 0.0;
		}
	}
	m_moveOut.write();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Motion::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Motion::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Motion::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Motion::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Motion::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void MotionInit(RTC::Manager* manager)
  {
    coil::Properties profile(motion_spec);
    manager->registerFactory(profile,
                             RTC::Create<Motion>,
                             RTC::Delete<Motion>);
  }
  
};


