﻿#ifndef TROBOT_SERIAL_IO_LINUX_HPP
#define TROBOT_SERIAL_IO_LINUX_HPP

#include "TRobotSerialIOIface.hpp"

#include "TRobotUtil.hpp"
#include <deque>
#include <pthread.h>


namespace TRobot
{
  //////////////////////////////////////////////////////////////////////
  /*!
   *  @brief Tロボットベースとのシリアル通信のためのクラス
   *
   *  @copybrief
   *  
   */
  class SerialIOLinux : public SerialIOIface
  {
  public:
    SerialIOLinux();
    virtual ~SerialIOLinux();
    virtual RetCode start(const char *port);
    virtual RetCode stop();
    virtual RetCode sendWheelRotVel(int leftRotVel, int rightRotVel);
    virtual RetCode receiveSensorData(BaseSensorData& sensorData);
    bool isRunning(); 
  
  private:
    void requestStop();
    bool stopRequested();
    RetCode receiveExactNumBytes(int fd, unsigned char *buf, size_t numBytes);
    RetCode sendExactNumBytes(int fd, unsigned char *buf, size_t numBytes);
    void *recvThreadFunc();
    static void *recvThreadFuncWrapper(void *arg);
    void *sendThreadFunc();
    static void *sendThreadFuncWrapper(void *arg);
    void pushSensorData(const BaseSensorData& sensorData);
    void popSensorData(BaseSensorData& sensorData);
    bool sensorDataQueueEmpty();
    void clearSensorDataQueue();

    void pushCmdPair(const RotVelCmdPair& cmdPair);
    void popCmdPair(RotVelCmdPair& cmdPair);
    bool cmdPairQueueEmpty();
    void clearCmdPairQueue();
    
    

    void seekToNextHeader(unsigned char *buf, size_t bufLen);
    
    int m_portFd; //!< シリアルポートのファイルディスクリプタ
    bool m_stopRequested; //!< 通信停止要求フラグ 
    bool m_isRunning; //!< 通信中か否かを表すフラグ 
    pthread_t m_recvThrId; //!< 受信スレッドのスレッドID
    pthread_t m_sendThrId; //!< 送信スレッドのスレッドID
    
    pthread_mutex_t m_stopReqLock; //!< 通信停止要求に関する排他処理のためのmutexオブジェクト
    pthread_mutex_t m_senDataQLock; //!< センサデータのキューに関する排他処理のためのmutexオブジェクト
    pthread_mutex_t m_rotVelCmdQLock; //!< 回転速度指令値のキューに関する排他処理のためのmutexオブジェクト

    pthread_cond_t m_condFirstSendDone; //!< 送信スレッドと受信スレッドの待ち合わせに用いる
    
    std::deque<BaseSensorData> m_sensorDataQueue; //!< センサデータのキュー
    std::deque<RotVelCmdPair> m_rotVelCmdQueue; //!< 回転速度指令値のキュー
  };
} // namespace TRobot
#endif // TROBOT_SERIAL_IO_LINUX_HPP
