﻿#ifndef TROBOT_SERIAL_IO_WINDOWS_HPP
#define TROBOT_SERIAL_IO_WINDOWS_HPP

#include "TRobotSerialIOIface.hpp"
#include "TRobotSensorData.hpp"
#include "TRobotUtil.hpp"

#include <deque>

#include <windows.h>


namespace TRobot
{
  //////////////////////////////////////////////////////////////////////
  /*!
   *  @brief Tロボットベースとのシリアル通信のためのクラス
   *
   *  @copybrief
   *  
   */
  class SerialIOWindows : public SerialIOIface
  {
  public:
    SerialIOWindows();
    virtual ~SerialIOWindows();
    virtual RetCode start(const char *portName);
    virtual RetCode stop();
    virtual RetCode sendWheelRotVel(int leftRotVel, int rightRotVel);
    virtual RetCode receiveSensorData(BaseSensorData& sensorData);
    bool isRunning(); 
  
  private:
    void requestStop();
    bool stopRequested();
    RetCode receiveExactNumBytes(HANDLE port, unsigned char *buf, size_t numBytes);
    RetCode sendExactNumBytes(HANDLE port, unsigned char *buf, size_t numBytes);
    DWORD recvThreadFunc(LPVOID params);
    DWORD recvThreadFuncMod(LPVOID params);
    
    static DWORD WINAPI recvThreadFuncWrapper(LPVOID params);
    
    void sendThreadFunc();
    static DWORD WINAPI sendThreadFuncWrapper(LPVOID params);
    
    
    void pushSensorData(const BaseSensorData& sensorData);
    void popSensorData(BaseSensorData& sensorData);
    bool sensorDataQueueEmpty();
    void clearSensorDataQueue();

    void pushCmdPair(const RotVelCmdPair& cmdPair);
    void popCmdPair(RotVelCmdPair& cmdPair);
    bool cmdPairQueueEmpty();
    void clearCmdPairQueue();
    
    
    void seekToNextHeader(unsigned char *buf, size_t bufLen);

    HANDLE m_commPort; //!< シリアルポートのハンドル
    HANDLE m_recvThrHandle; //< 受信スレッドのハンドル
    HANDLE m_sendThrHandle; //< 送信スレッドのハンドル
    
    bool m_stopRequested; //!< 通信停止要求フラグ 
    bool m_isRunning; //!< 通信中か否かを表すフラグ 
    
    CRITICAL_SECTION m_stopReqLock; //!< 通信停止要求に関する排他処理のためのオブジェクト
    CRITICAL_SECTION m_senDataQLock; //!< センサデータのキューに関する排他処理のためのオブジェクト
    CRITICAL_SECTION m_rotVelCmdQLock; //!< 回転速度指令値のキューに関する排他処理のためのオブジェクト
    
    std::deque<BaseSensorData> m_sensorDataQueue; //!< センサデータのキュー
    std::deque<RotVelCmdPair> m_rotVelCmdQueue; //!< 回転速度指令値のキュー
  };
} // namespace TRobot

#endif // TROBOT_SERIAL_IO_WINDOWS_HPP
